package ui.controller;

import java.util.ArrayList;

import Interfaces.CategoryListener;
import Interfaces.ObjectListener;
import classes.CpsEdge;
import classes.CpsObject;
import ui.model.Model;

public class CanvasController {

	private Model MODEL;
	private MultiPurposeController mpC;

	public CanvasController(Model model, MultiPurposeController mp) {
		this.MODEL = model;
		this.mpC = mp;
	}

	/**
	 * Add an CpsObject to the model and notify the ObjectListener for update.
	 * 
	 * @param object
	 *            CpsObject to be added.
	 */
	public void addObject(CpsObject object) {
		MODEL.getCvsObjIdx().put(object.getID(), MODEL.getObjectsOnCanvas().size());
		MODEL.getObjectsOnCanvas().add(object);
		notifyObjListeners();
	}

	/**
	 * 
	 * @param object
	 */
	public void addNewObject(CpsObject object) {
		object.setSav("CVS");
		object.setConnections(new ArrayList<CpsEdge>());
		addObject(object);
	}

	public void addObjectListener(ObjectListener objLis) {
		MODEL.getObjectListeners().add(objLis);
	}

	/**
	 * notifies all listeners about changes in the Canvas
	 */
	public void notifyObjListeners() {
		for (ObjectListener l : MODEL.getObjectListeners()) {
			l.onChange(MODEL.getObjectsOnCanvas());
		}
	}

	/**
	 * Deletes an CpsObject on the Canvas and its connections
	 * 
	 * @param CpsObject
	 */
	public void deleteObjectOnCanvas(CpsObject obj) {
		CpsEdge e = null;
		for (CpsObject cps : MODEL.getObjectsOnCanvas()) {
			for (CpsEdge p : cps.getConnections()) {
				if (p.getA() == obj || p.getB() == obj) {
					e = p;
				}
			}
			cps.getConnectedTo().remove(e);
			MODEL.getEdgesOnCanvas().remove(e);
		}
		mpC.decIdx(obj.getID(), MODEL.getCvsObjIdx());
		MODEL.getCvsObjIdx().remove(obj.getID());
		MODEL.getObjectsOnCanvas().remove(obj);

	}

	/**
	 * @param objectsOnCanvas
	 *            the objectsOnCanvas to set
	 */
	public void addEdgeOnCanvas(CpsEdge edge) {
		MODEL.getEdgesOnCanvas().add(edge);
	}

	/**
	 * @param edgesOnCanvas
	 *            the edge to remove
	 */
	public void removeEdgesOnCanvas(CpsEdge edge) {
		MODEL.getEdgesOnCanvas().remove(edge);
	}

}
