package ui.controller;

import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.ArrayList;

import Interfaces.CategoryListener;
import classes.Category;
import classes.CpsEdge;
import classes.CpsObject;
import classes.HolonElement;
import classes.HolonObject;
import ui.model.Model;

public class Control {

	private Model MODEL;

	private ActionListener actionListener;

	private final MultiPurposeController multiPurposeController;
	private final CategoryController categoryController;
	private final ObjectController objectController;
	private final CanvasController canvasController;
	private final GlobalController globalController;
	private final StoreController storeController;
	private final LoadController loadController;

	public Control(Model model) {
		this.MODEL = model;

		this.multiPurposeController = new MultiPurposeController(MODEL);
		this.categoryController = new CategoryController(MODEL, multiPurposeController);
		this.objectController = new ObjectController(MODEL, multiPurposeController);
		this.canvasController = new CanvasController(MODEL, multiPurposeController);
		this.globalController = new GlobalController(MODEL);
		this.storeController = new StoreController(MODEL);
		this.loadController = new LoadController(MODEL, categoryController, canvasController, objectController,
				multiPurposeController);

	}

	/* Operations for searching */

	public CpsObject searchByID(int ID) {
		return multiPurposeController.searchByID(ID);
	}

	public CpsObject searchCategoryObject(String category, String object) {
		return multiPurposeController.searchCategoryObject(multiPurposeController.searchCategory(category), object);
	}

	/* Operations for Categories and Objects */
	public void addCategory(String cat) {
		categoryController.addNewCategory(cat);
	}

	public void addObject(Category cat, String obj, ArrayList<HolonElement> ele, String img) {
		categoryController.addNewHolonObject(cat, obj, ele, img);
	}

	public void addTransformer(Category cat, String obj) {
		categoryController.addNewHolonTransformer(cat, obj, "/Images/transformer-1.png");
	}

	public void addSwitch(Category cat, String obj) {
		categoryController.addNewHolonSwitch(cat, obj, "/Images/switch-off.png");
	}

	public Category searchCategory(String cat) {
		return multiPurposeController.searchCategory(cat);
	}

	public void deleteCategory(String cat) {
		categoryController.deleteCategory(cat);
	}

	public void delObjectCategory(String cat, String obj) {
		categoryController.deleteObject(cat, obj);
	}

	/* Operations for Canvas */

	public void AddEdgeOnCanvas(CpsEdge edge) {
		canvasController.addEdgeOnCanvas(edge);
	}

	public void removeEdgesOnCanvas(CpsEdge edge) {
		canvasController.removeEdgesOnCanvas(edge);
	}

	public void setSelecteEdge(CpsEdge edge) {
		MODEL.setSelectedEdge(edge);
	}

	public void addObjectCanvas(CpsObject object) {
		canvasController.addNewObject(object);
	}

	public void setSelectedObjectID(int id) {
		objectController.setSelectedObjectID(id);
	}

	public void delCanvasObject(CpsObject obj) {
		canvasController.deleteObjectOnCanvas(obj);
	}

	/* Operations for Objects and Elements */
	public void addElementCanvasObject(int ID, String ele, int amount, float energy) {
		objectController.addNewElementIntoCanvasObject(ID, ele, amount, energy);
	}

	public void addElementCategoryObject(String catName, String objName, String eleName, int amount, float energy) {
		objectController.addNewElementIntoCategoryObject(catName, objName, eleName, amount, energy);
	}

	public void deleteElementCanvas(int id, String element) {
		objectController.deleteElementInCanvas(id, element);
	}

	public void deleteElementCanvas(HolonObject obj, HolonElement ele) {
		objectController.deleteElement(obj, ele);
	}

	/* Global Operations */
	public int getScale() {
		return globalController.getScale();
	}

	public int getScaleDiv2() {
		return globalController.getScaleDiv2();
	}

	public void setScale(int s) {
		globalController.setScale(s);
	}

	public void setCurIteration(int cur_it) {
		globalController.setCurIteration(cur_it);
	}

	/* Operations for Loading and Storing */
	public void saveFile(String path) throws IOException {
		storeController.writeJSONFile(path);
	}

	public void loadFile(String path) throws IOException {
		loadController.readJSON(path);
	}

	////////// etc
	public void initListener(CategoryListener catLis) {
		categoryController.addCategoryListener(catLis);
	}

	/**
	 * Getter for Model
	 * 
	 * @return
	 */
	public Model getModel() {
		return MODEL;
	}

}
